#!/bin/bash
# From Travis 
BIN_NAME="enable_vrx"
CHANNEL=161
MODE="11AHE20"
KER_VER=`uname -r`

declare -A channel_array
channel_array=(
	[144]="144  - 5720 - 20"
	[149]="149  - 5745 - 20"
	[153]="153  - 5765 - 20"
	[157]="157  - 5785 - 20"
	[161]="161  - 5805 - 20"
	[165]="165  - 5825 - 20"
	[169]="169  - 5845 - 20"
	[173]="173  - 5865 - 20"
)

print_info(){
	echo "$BIN_NAME : INFO : $1"
}
print_error(){
	echo "$BIN_NAME : ERROR : $1"
}
print_channels(){
	echo "CHAN - BW  - FREQ"
	for channel in $(echo "${!channel_array[@]}" | tr ' ' '\n' | sort -n); do
		echo "${channel_array[$channel]}"
	done
}
print_usage(){
	echo "Usage:"
	echo "  $BIN_NAME -h                      - show help"
	echo "  $BIN_NAME -c <channel_number>     - set channel"
	echo ""
	print_channels
	echo ""
	exit 1
}


# Check if the input has the correct format
if [[ "$1" == "-c" && "$2" =~ ^[0-9]+$ ]]; then
	CHANNEL=$2
	# Validate if the channel exists in the array
	if [[ -n "${channel_array[$CHANNEL]}" ]]; then
		print_info "Channel $CHANNEL is valid"
	else
		print_info "Invalid channel: $CHANNEL"
		print_channels
		exit 1
	fi
else
	print_usage
fi




if lspci | grep -q "Qualcomm Device"; then
	print_info "Qualcomm PCI Device found"
else
	print_error "Qualcomm PCI Device not found"
	exit 1
fi

# Function to check if a module is loaded and load it if not
load_module() {
	local name="$1"
	local path="$2"

	# Check if the module is already loaded
	if lsmod | grep -q "^${name} "; then
		print_info "Module ${name} is already loaded."
	else
		print_info "Loading module ${name}..."
		insmod "${path}" || print_error "Failed to load module ${name}"
	fi
}

load_module "qca_cnss2" "/lib/modules/$KER_VER/extra/qca_cnss2.ko"
load_module "mem_manager" "/lib/modules/$KER_VER/extra/mem_manager.ko"
load_module "qdf" "/lib/modules/$KER_VER/extra/qdf.ko"
load_module "umac" "/lib/modules/$KER_VER/extra/umac.ko"
load_module "qca_spectral" "/lib/modules/$KER_VER/extra/qca_spectral.ko"
load_module "qca_ol" "/lib/modules/$KER_VER/extra/qca_ol.ko"
load_module "wifi_3_0" "/lib/modules/$KER_VER/extra/wifi_3_0.ko"

#sleep 1

cfg80211tool wifi0 enable_ol_stats 1

# Ceate a new Virtual Access Point interface for monitor mode called ath0
wlanconfig ath0 create wlandev wifi0 wlanmode specialvap -cfg80211

echo "the following shows invalid arg"
# Create VAP interface (ath0) on the physical wireless device (wifi0)
iw dev wifi0 interface add ath0 type __ap

#cfg80211tool ath0 mcast_rate_control 1

#cfg80211tool ath0 mode "$MODE"

cfg80211tool ath0 mode 11AHE20
#cfg80211tool ath0 channel "$CHANNEL" 2

cfg80211tool ath0 channel "$CHANNEL" 2

# set and hide SSID
cfg80211tool ath0 ssid Openwrt_specialvap
cfg80211tool ath0 hide_ssid 0

# Wireless Distribution System (WDS) disabled
cfg80211tool ath0 wds 0
# disable backhaul, interface will not be used for handling traffic between access points
cfg80211tool ath0 backhaul 0

cfg80211tool ath0 en_6g_sec_comp 1

cfg80211tool ath0 uapsd 1

cfg80211tool ath0 stafwd 0

#cfg80211tool ath0 mscs 0

cfg80211tool ath0 ap_bridge 1

echo "the following two show errors"
iwpriv ath0 mcast_rate 103200
iwpriv ath0 bcast_rate 103200

#iwpriv ath0 qdf_cv_lvl 0x68000a

ifconfig ath0 up

dmesg | grep "send_vdev_start_cmd_tlv" | tail -n 1
dmesg | grep "htt_h2t_full_mon_cfg" | tail -n 1